/*
 *Copyright (C) 2001  LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdlib.h>
#include <iostream>
#include "buffer.hh"

buffer::buffer(int nb_pictures, int size,bool free) :
  number_of_pictures(nb_pictures),
  buffer_size(size),
  first_image(0),
  last_image(size),
  current_loaded_image(0),
  current_process_image(0),
  last_position_get_image(0),
  position_get_image(0),
  must_free_image(free),
  calcul(0),
  mutex_last_image              (new wxMutex()),
  mutex_first_image             (new wxMutex()),
  mutex_loaded_image            (new wxMutex()),
  mutex_last_position           (new wxMutex()),
  load_thread_you_can_load      (new wxCondition()),
  thread_process_you_can_process(new wxCondition())
{
  images.resize(number_of_pictures+1);
  images_process.resize(number_of_pictures+1);
  fill(images.begin(),images.end(),(wxImage *)0);
  fill(images_process.begin(),images_process.end(),(wxImage *)0);
  //cout << "tampon debut " << endl;
}

// TODO : homogenity

void buffer::delete_image(int position)
{
  if(position>=0 && position<number_of_pictures)
  {
    delete_process_image(position);
    if(images[position]!=0)
    {
      if(must_free_image)
      {
	free(images[position]->GetData());
	//cout << "free" << endl;
      }
      //cout << "tampon position" << position << " " << &images[position] << endl;
      delete images[position];
      images[position]=0;
    }
  }
}

void buffer::delete_process_image(int position)
{
  if(position>=0 && position<number_of_pictures)
  {
    delete images_process[position];
    images_process[position]=0;
  }
}

void buffer::delete_all_images()
{
  for(int i=get_first_image()-1;
      i<=get_last_image()+1;i++)
  {
    delete_image(i);
  }
}

void buffer::delete_all_images_process()
{
  for(int i=get_first_image()-1;
      i<=get_last_image()+1;i++)
  {
    delete_process_image(i);
  }
}

void buffer::reinitialize_buffer_range(type_of_current_reading current_reading,
				       int current_position)
{
  delete_all_images();
  set_last_position(current_position);
  switch(current_reading)
  {
  case PLAY_FORWARD:
    set_first_image(current_position);
    set_last_image(current_position+get_buffer_size());
    break;

  case PLAY_BACKWARD:
    set_first_image(current_position-get_buffer_size());
    set_last_image(current_position);
    break;

  case IMAGE_BACKWARD:				  //TODO : buffer also in this case
  case IMAGE_FORWARD :
    set_first_image(current_position);
    set_last_image(current_position);
    break;

  default:
    break;
  }
  set_current_loaded_image(current_position);
  set_current_process_image(current_position);
}

void buffer::map_array_for_debug() const
{
  cout << "first_position   : " << get_first_image() << endl;
  cout << "last_position    : " << get_last_image() << endl;
  cout << "current load     : " << get_current_loaded_image() << endl;
  cout << "mappage du tableau : "  << endl;
  for(int i=0;i<=number_of_pictures;i++)
  {
    cout << "i:";
    if(images[i]!=0)
      cout << i <<" ";
    else
      cout << "0 ";
  }
  cout << "fin mappage" << endl;
}

void buffer::map_array_process_for_debug() const
{
  cout << "first_position   : " << get_first_image() << endl;
  cout << "last_position    : " << get_last_image() << endl;
  cout << "current process     : " << get_current_process_image() << endl;
  cout << "mappage du tableau : "  << endl;
  for(int i=0;i<=number_of_pictures;i++)
  {
    cout << "i:";
    if(images_process[i]!=0)
      cout << i <<" ";
    else
      cout << "0 ";
  }
  cout << "fin mappage" << endl;
}

void buffer::put_process_image(int position, wxImage& image)
{
  if(images_process[position]==0)
  {
    //cout << "insertion" << endl;
    images_process[position]=&image;
  }
  else
  {
    //cout << "images_process non vide" << endl;
    // TODO : reactivate this code that is currently in thread....
    /*
    wxImage *im;
    im=images_process[position];
    //delete_process_image(position);
    images_process[position]=&image;
    free(im->GetData());
    delete im;
    */
    images_process[position]=&image;
  }
}

void buffer::put_image(int position, wxImage& image)
{
  if(images[position]==0)
  {
    //cout << "insertion" << endl;
    images[position] = &image;
  }
  else
  {
    //cout << "vide" << endl;
    delete_image(position);
    images[position] = &image;
  }
}

wxImage* buffer::get_image(int position)
{
  return images[position];
}

wxImage* buffer::get_process_image(int position)
{
  return images_process[position];
}

wxImage* buffer::get_image(int position,
			   type_of_current_reading current_reading)
{
  switch(current_reading)
  {
  case PLAY_FORWARD:
  case PLAY_BACKWARD:
    //map_array_process_for_debug();
    //map_array_for_debug();
    if(get_last_position() == position-current_reading 	||
       get_last_position() == position)
    {
      set_last_position(position);

      if(images[position]!=0)
      {
	if(images_process[position]!=0 || calcul==0)
	{
	  //cout << "buf process!=0" << endl;

	  delete_image(first_image-current_reading);
	  set_first_image(first_image+current_reading);
	  set_last_image(last_image+current_reading);
	  load_thread_you_can_load->Signal();
	  if(calcul==0)
	  {
	    return images[position];
	  }
	  else
	  {
	    return images_process[position];
	  }
	}
	else
	{
	  //cout << "dans process == 0" << endl;
	  load_thread_you_can_load->Signal();
	  if(calcul!=0)
	  {
	    thread_process_you_can_process->Signal();
	    return images_process[position];
	  }
	  else
	  {
	    return images[position];
	  }
	}
      }
      else
      {
	if(position==get_current_loaded_image())
	{
	  load_thread_you_can_load->Signal();
	  return images[position];
	}
	else
	{
	  reinitialize_buffer_range(current_reading,position);
	  return 0;
	}
      }
    }
    else
    {
      reinitialize_buffer_range(current_reading,position);
      return 0;
    }
  break;

  case IMAGE_BACKWARD:
  case IMAGE_FORWARD:
    //map_array_for_debug();
    if(images[position-1] != 0 	||
       images[position+1] != 0 	||
       images[position]	  == 0)
    {
      reinitialize_buffer_range(current_reading,position);
      load_thread_you_can_load->Signal();
    }
    if(calcul==1 || calcul==2)
    {
      if(images_process[position]==0 || calcul==2)
      //if(calcul==2)
      {
	thread_process_you_can_process->Signal();
	cout << "changement de zoom" << endl;
      }
      //cout << "dans 1" << endl;
      return images_process[position];
    }
    else
    {
      return images[position];
    }
    break;
  case UN_INITIALIZED:
  case STOP:
    break;
  }
  return 0;
}


int buffer::get_number_of_pictures() const
{
  return number_of_pictures;
}

void buffer::set_calcul(int foo)
{
  // calcul 0 si pas de calcul  ffectuer
  // calcul 1 si calcul  ffectuer
  // calcul 2 si changement du type de calcul
  calcul = foo;
}

int buffer::get_calcul() const
{
  return calcul;
}

int buffer::get_buffer_size() const
{
  return buffer_size;
}

int buffer::get_first_image() const
{
  wxMutexLocker lock(*mutex_first_image);
  return first_image;
}

int buffer::get_last_image() const
{
  wxMutexLocker lock(*mutex_last_image);
  return last_image;
}

int buffer::get_current_loaded_image() const
{
  wxMutexLocker lock(*mutex_loaded_image);
  return current_loaded_image;
}

int buffer::get_current_process_image() const
{
  return current_process_image;
}

bool buffer::image_not_existing(int position) const
{
  return images[position]==0;
}

void buffer::set_first_image(int position)
{
  // TODO : use wxMutexLocker ?
  mutex_first_image->Lock();
  if(position>=0) // TODO : simplify by projection
  {
    if(position<number_of_pictures)
      first_image = position;
    else
      first_image = number_of_pictures-1;
  }
  else
  {
    first_image=0;
  }
  mutex_first_image->Unlock();
}

void buffer::set_last_image(int position)
{
  mutex_last_image->Lock();
  if(position<number_of_pictures)
    last_image=position;
  else
    last_image=number_of_pictures-1;
  mutex_last_image->Unlock();
}

void buffer::set_current_loaded_image(int position)
{
  mutex_loaded_image->Lock();
  if(position<get_first_image())
  {
    current_loaded_image=get_first_image();
  }
  else if(position>get_last_image())
  {
    current_loaded_image=get_last_image();
  }
  else
  {
    current_loaded_image=position;
  }
  mutex_loaded_image->Unlock();
}

void buffer::set_current_process_image(int position)
{
  current_process_image=position;
}

int buffer::get_last_position() const
{
  wxMutexLocker lock(*mutex_last_position);
  return last_position_get_image;
}

void buffer::set_last_position(int position)
{
  mutex_last_position->Lock();
  last_position_get_image=position;
  mutex_last_position->Unlock();
}

void buffer::initialize_condition()
{
  if(load_thread_you_can_load!=0)
  {
    delete load_thread_you_can_load;
  }
  load_thread_you_can_load=new wxCondition();
}

void buffer::initialize_condition_process()
{
  if(thread_process_you_can_process!=0)
  {
    delete thread_process_you_can_process;
  }
  thread_process_you_can_process=new wxCondition();
}
