/*
 *Copyright (C) 2001  LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef buffer__
#define buffer__

#include <vector>

#include <wx/image.h>
#include <wx/wx.h>

#include "common_type.hh"
#include "sequence.hh"

using namespace std;

class buffer
{
private:
  int number_of_pictures;			  // in the whole sequence
  int buffer_size;
  int first_image;
  int last_image;
  int current_loaded_image;
  int current_process_image;
  int last_position_get_image;
  int position_get_image;
  bool must_free_image;
  int calcul;					  // TODO : enum, rename

  vector<wxImage*> images;
  vector<wxImage*> images_process;

  void delete_image(int position);
  void delete_process_image(int position);
  void delete_all_images();
  void delete_all_images_process();
  void reinitialize_buffer_range(type_of_current_reading current_reading,
				 int current_position);
  void map_array_for_debug() const;
  void map_array_process_for_debug() const;

public:
  wxMutex* mutex_last_image;
  wxMutex* mutex_first_image;
  wxMutex* mutex_loaded_image;
  wxMutex* mutex_last_position;

  wxCondition* load_thread_you_can_load;
  wxCondition* thread_process_you_can_process;

  buffer(int nb_pictures, int size, bool free);
  //TODO : destructor with delete

  void put_image(int position, wxImage& image);
  void put_process_image(int position, wxImage& image);
  bool image_not_existing(int position) const;
  void initialize_condition();
  void initialize_condition_process();

  wxImage* get_image(int position);
  wxImage* get_process_image(int position);
  wxImage* get_image(int position,
		     type_of_current_reading current_reading);
  int get_number_of_pictures() const;
  int get_buffer_size() const;
  int get_first_image() const;
  int get_last_image() const;
  int get_current_loaded_image() const;
  int get_current_process_image() const;
  int get_calcul() const;
  int get_last_position() const;

  void set_first_image(int position);
  void set_last_image(int position);
  void set_current_loaded_image(int position);
  void set_current_process_image(int position);
  void set_last_position(int position);
  void set_calcul(int foo);
};

#endif
