/*
 *Copyright (C) 2001  LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <string>
#include <wx/wxprec.h>

#include "my_application.hh"
#include "my_main_frame.hh"
#include "my_frame.hh"
#include "sequence.hh"
#include "sequence_yuv.hh"
#include "sequence_wxformat.hh"
#include "sequence_pgm.hh"
#include "my_dialog_yuv.hh"

IMPLEMENT_APP(my_application)

using namespace std;

const wxString version_string="MultiVideo version 0.1.0";

my_application::my_application()
{
  her_main_frame = new my_main_frame(0,
				     version_string,
				     wxPoint(0,0),
				     wxSize(230,320),this);
}

my_application::~my_application()
{
  //delete her_main_frame;
}

/* method launched at the program start (like main) */
bool my_application::OnInit(void)
{
  // tells wxWindow about unsupported image formats
  initialize_image();

  // main window with global controls
  create_window();

  // display it
  paint_window();

  analyse_command_line(argc,argv);
  return TRUE;
}

void my_application::create_window()
{
  her_main_frame->create_video_control_tbar();
  her_main_frame->create_speed_controls();
  her_main_frame->create_menu();
}

void my_application::paint_window()
{
  her_main_frame->SetStatusText(version_string);
  her_main_frame->Show(TRUE);
}

void my_application::insert_frame(my_frame&  frame)
{
  her_frames.push_back(&frame);
}

void my_application::remove_frame(my_frame& frame)
{
  int i=0;
  bool find=false;

  // TODO use find from stdc++

  while(i<(int)her_frames.size() || !find)
  {
    if(her_frames[i]==&frame)
    {
      //cout << "trouve" << endl;
      her_main_frame->TimerStop();
      delete her_frames[i];
      her_main_frame->TimerStart(her_main_frame->get_speed());
      her_frames.erase(her_frames.begin()+i);
      find=true;
    }
    i++;
  }

  // TODO call the existing function
  my_frame *frame_greater(0);
  my_frame *frame_minus(0);
  if(her_frames.size()==0)
  {
    frame_greater=0;
  }
  else
  {
    i=1;
    frame_greater=her_frames[0];
    frame_minus=her_frames[0];
    //cout << "nbframes" << her_frames.size() << endl;
    while(i<(int)her_frames.size())
    {
      if(her_frames[i]->get_sequence()->get_nb_pictures()>
	 frame_greater->get_sequence()->get_nb_pictures())
      {
	//cout << "sup" << endl;
	frame_greater=her_frames[i];
      }
      if(her_frames[i]->get_sequence()->get_start_position()<
	 frame_minus->get_sequence()->get_start_position())
      {
	frame_minus=her_frames[i];
      }
      i++;
    }
    her_main_frame->get_slider_position()->SetRange(frame_minus->get_sequence()->get_start_position(),
						    frame_greater->get_sequence()->get_nb_pictures()-1);
    her_main_frame->set_position_max(frame_greater->get_sequence()->get_nb_pictures());
  }
  her_main_frame->set_greater_frame(frame_greater);
  her_main_frame->set_minus_frame(frame_minus);
}

void my_application::initialize_image() const
{
  wxImage::AddHandler(new wxJPEGHandler);
  wxImage::AddHandler(new wxPNGHandler);
  wxImage::AddHandler(new wxPNMHandler);
  wxImage::AddHandler(new wxBMPHandler);
}

void my_application::new_frame(sequence& seq, int buffer_size)
{
  my_frame *frame = new my_frame(0, seq.get_name(0),
				 wxPoint(300+(50*her_frames.size()),
				 50*her_frames.size()),
				 wxDefaultSize,*her_main_frame,seq,
				 buffer_size);
 insert_frame(*frame);

 // Update min and max of sequences

 if(her_main_frame->get_greater_frame()==0)
   her_main_frame->set_greater_frame(frame);
 else if(her_main_frame->get_greater_frame()->get_sequence()->get_nb_pictures()
	 < frame->get_sequence()->get_nb_pictures())
 {
   //cout << "INFERIEUR" << endl;
   her_main_frame->set_greater_frame(frame);
 }

 if(her_main_frame->get_minus_frame()==0)
   her_main_frame->set_minus_frame(frame);
 else if(her_main_frame->get_minus_frame()->get_sequence()->get_start_position()>
	 frame->get_sequence()->get_start_position())
 {
   her_main_frame->set_minus_frame(frame);
 }

 //update main slider

 her_main_frame->get_slider_position()->
   SetRange(her_main_frame->get_minus_frame()->get_sequence()->get_start_position(),
	    her_main_frame->get_greater_frame()->get_sequence()->get_nb_pictures()-1);
 her_main_frame->set_position_max(frame->get_sequence()->get_nb_pictures());
}

int my_application::get_nb_frames() const
{
  return her_frames.size();
}

my_frame* my_application::get_frame(int nb) const
{
  return her_frames[nb];
}

void my_application::find_sequence_extension_and_create_new_frame(vector<wxString> filenames,
								  int width, int height, int buffer_size)
{
  sequence *seq;
  wxString extension=filenames[0].AfterLast('.');
  cout <<"file: "<<filenames[0]<<" extension: "<<extension<<endl;
  if(extension.Contains("yuv"))
  {
    seq=new sequence_yuv(filenames, width, height);
  }
  else if(extension.Contains("pgm"))
  {
    seq=new sequence_pgm(filenames);
  }
  else
  {
    seq=new sequence_wxformat(filenames);
  }
  new_frame(*seq,buffer_size);
}

void my_application::analyse_command_line(int argc, char **argv)
{
  int i=1;
  bool seq=true;
  long buffer_size;
  buffer_size=10;
  vector<wxString>vector_of_filename_sequence;
  if(wxString(argv[i])=="-seq" ||
     wxString(argv[i])=="--sequence")
  {
    while(seq && i<argc)
    {
      long width=0;
      long height=0;
      vector_of_filename_sequence.clear();
      seq=false;
      while( i+1 <argc && !seq)
      {
	if(wxString(argv[i+1])=="-seq" ||
	   wxString(argv[i+1])=="--sequence")
	{
	  seq=true;
	}
	else if(wxString(argv[i+1])=="--size" ||
		wxString(argv[i+1])=="-s")
	{
	  i++;
	  ((wxString(argv[i+1])).BeforeFirst('x')).ToLong(&width);
	  ((wxString(argv[i+1])).AfterFirst('x')).ToLong(&height);
	  if(width % 2 != 0 || height %2 !=0)
	  {
	    cout << "taille de la sequence impaire ! " << endl;
	    width=0;
	    height=0;
	  }
	}
	else if(wxString(argv[i+1])=="--buffer-size" ||
		wxString(argv[i+1])=="-b")
	{
	  i++;
	  (wxString(argv[i+1])).ToLong(&buffer_size);
	  cout << "buffer_size : " << buffer_size<<endl;
	}
	else if(wxString(argv[i+1])=="--play-mode")
	{
	  i++;
	  // not implemented
	}
	else
	{
	  //printf("fichier %s\n",argv[i+1]);
	  vector_of_filename_sequence.push_back(wxString(argv[i+1]));
	}
	i++;
      }
	if((width==0 || height==0) && vector_of_filename_sequence[0].Contains("yuv"))
	{
	  //cout << "set_size" << endl;
	  set_size_with_dialog(width, height,vector_of_filename_sequence[0]);
	}
	//cout << " find_sequence" << endl;
	find_sequence_extension_and_create_new_frame(vector_of_filename_sequence,
						     width,height,buffer_size);
    }
    for(int i=0;i<(int)her_frames.size();i++)
    {
      her_frames[i]->set_current_position(her_main_frame->get_minus_frame()
					  ->get_sequence()->get_start_position());
    }
    her_main_frame->OnPlayForward();
  }
  else if(wxString(argv[i])=="--version" ||
	  wxString(argv[i])=="-v")
  {
    cout << version_string << endl;
    cout << endl;
    cout << "This is free software."<< endl;
    cout << "There is NO warranty"<<endl;
    cout << "not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."<<endl;
    her_main_frame->OnQuit();
  }
  else if(wxString(argv[i])=="--help" ||
	  wxString(argv[i])=="-h")
  {
    cout << "Usage: multivideo [-seq file(s) [sequence options]]..." << endl;
    cout << "See USAGE file for details." << endl;
  }
}

void my_application::set_size_with_dialog(long& width, long& height,
					  wxString sequence_name)
{
  //cout << "In set_size" << endl;
  my_dialog_yuv dialog_yuv(0, sequence_name.AfterLast('/'),
			   wxPoint(100, 100), wxSize(300, 200));
  if(dialog_yuv.ShowModal()==wxID_OK)
  {
    dialog_yuv.width.ToLong(&width);
    dialog_yuv.height.ToLong(&height);
    cout << "width : " << width << " height : " << height << endl;
  }
  if(width==0 || height==0)
  {
    width=352;
    height=288;
  }
}
