/*
 *Copyright (C) 2001   LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>

#include "my_frame.hh"
#include "my_thread_load.hh"
#include "my_thread_process.hh"
#include "sequence.hh"
#include "my_main_frame.hh"
#include "sequence_yuv.hh"
#include "sequence_wxformat.hh"
#include "sequence_pgm.hh"
#include "buffer.hh"
#include "my_dialog_save.hh"

my_frame::my_frame(wxFrame* parent, const wxString& title,
		   const wxPoint& pos, const wxSize& size,
		   my_main_frame& my_mother,sequence& seq,
		   int buffer_size) :
  wxFrame(parent,-1,title,pos,size),
  her_thread_load(0),
  her_thread_process(0),
  her_sequence(&seq),
  my_mother(&my_mother),
  her_buffer(new buffer(her_sequence->get_nb_pictures(),
			buffer_size,
			get_sequence()->must_free_data_image())),
  is_readed(0),
  local_type_of_current_reading(IMAGE_FORWARD),
  zoom(1),
  m_slider(0),
  draw_frame(0)
{
  create_local_video_tbar();
  create_threads();
  SetSize(her_sequence->get_width(),
	  her_sequence->get_height()+40);
  this->Show();
}

my_frame::~my_frame()
{
  her_thread_load->should_continue=false;
  her_thread_process->should_continue=false;
  her_buffer->load_thread_you_can_load->Signal();
  her_buffer->thread_process_you_can_process->Signal();
  //her_thread_process->Pause();
  her_thread_process->Wait();
  her_thread_load->Wait();
  delete her_thread_process;
  delete her_thread_load;
  delete her_buffer;
}

void my_frame::create_threads()
{
  her_thread_load=new my_thread_load(*this,*her_buffer,*her_sequence);
  her_thread_process=new my_thread_process(*this,*her_buffer,*her_sequence);
  her_thread_load->Create();
  her_thread_process->Create();
  her_thread_load->Run();
  her_thread_process->Run();
}

void my_frame::create_local_video_tbar()
{
  long style = wxNO_BORDER |wxEXPAND| wxTB_FLAT | wxTB_DOCKABLE | wxTB_HORIZONTAL;
  this->SetAutoLayout(TRUE);
  m_slider = new wxSlider(this, ID_SLIDER,  her_sequence->get_start_position(),
			  her_sequence->get_start_position(),
			  her_sequence->get_nb_pictures(),
			  wxPoint(-1,-1),
			  wxSize(-1,-1),
			  style);

  wxButton *button_save=new wxButton(this,ID_BUTTON_SAVE,"grab");
  draw_frame=new wxScrolledWindow(this,-1,
				  wxPoint(11,-1),
				  wxSize(her_sequence->get_width(),
					 her_sequence->get_height()),
				  wxCAPTION | wxNO_BORDER,
				  "frame affichage");
  child_spinbutton_position = new wxSpinButton(this,ID_SPIN_BUTTON_POSITION);
  child_spinbutton_position->SetRange(/*get_sequence()->get_start_position()*/0,
				      get_sequence()->get_nb_pictures()-1);

  wxComboBox *zoom_button=new wxComboBox(this,ID_BUTTON_ZOOM);
  zoom_button->Append("1");
  zoom_button->Append("2");
  zoom_button->Append("3");

  wxLayoutConstraints *layout_static_spin=new wxLayoutConstraints;
  layout_static_spin->left.PercentOf (this, wxWidth, 0);
  layout_static_spin->width.PercentOf(this, wxWidth,1);
  layout_static_spin->top.SameAs  (this, wxTop, 10);
  child_spinbutton_position->SetConstraints(layout_static_spin);
  child_spinbutton_position->SetSize(-1,15,-1,10,wxSIZE_USE_EXISTING);

  wxLayoutConstraints *layout_static_slider=new wxLayoutConstraints;
  layout_static_slider->left.PercentOf (this, wxWidth, 0);
  layout_static_slider->left.RightOf   (child_spinbutton_position,18);
  layout_static_slider->top.SameAs     (this, wxTop, 3);
  layout_static_slider->width.PercentOf(this, wxWidth, 60);
  layout_static_slider->height.SameAs(zoom_button, wxHeight, 8);
  m_slider->SetConstraints(layout_static_slider);

  wxLayoutConstraints *layout_static_zoom=new wxLayoutConstraints;
  layout_static_zoom->left.PercentOf  (this, wxWidth, 0);
  layout_static_zoom->left.RightOf    (m_slider,18);
  layout_static_zoom->top.SameAs      (m_slider, wxTop,3);
  layout_static_zoom->width.PercentOf (this, wxWidth, 10);
  layout_static_zoom->height.PercentOf(this, wxHeight, 8);
  zoom_button->SetConstraints(layout_static_zoom);

  wxLayoutConstraints *layout_static_save=new wxLayoutConstraints;
  layout_static_save->left.RightOf   (zoom_button,5);
  layout_static_save->top.SameAs  (this, wxTop, 5);
  layout_static_save->width.PercentOf (this, wxWidth, 10);
  layout_static_save->height.PercentOf(this, wxHeight, 8);
  button_save->SetConstraints(layout_static_save);

  wxLayoutConstraints *layout_static_frame=new wxLayoutConstraints;
  layout_static_frame->left.PercentOf  (this, wxWidth, 0);
  layout_static_frame->top.Below       (m_slider, 10);
  layout_static_frame->width.PercentOf (this, wxWidth, 100);
  layout_static_frame->height.PercentOf(this, wxHeight, 90);
  draw_frame->SetConstraints(layout_static_frame);
}


void my_frame::draw_bitmap()
{
  wxClientDC dc(draw_frame);
  int temp_position=get_current_position();
  type_of_current_reading cr=get_type_of_current_reading();
  set_is_readed(false);
  set_slider_position(get_current_position());
  child_spinbutton_position->SetValue(get_current_position());
  wxImage* temp=her_buffer->get_image(get_current_position(), cr);

  if(temp!=0)					  // image in buffer
  {
    wxBitmap bitmap(temp->ConvertToBitmap());	  // done in a single thread
						  // otherwise use XInitThreads (but complicated)
    if(bitmap.Ok())
    {
      dc.DrawBitmap(bitmap,0,0);
    }
    set_new_current_position();
  }
  else						  // image not yet in buffer
  {
    //cout << "PAS AVANCE" << endl;
  }

  if(temp_position!=get_current_position())
  {
    set_is_readed(true);
  }

  // when we try to display an out of bound image, do as if it were already available
  if(this!=my_mother->get_greater_frame() && //TODO use start_...
     (get_current_position()>=her_sequence->get_nb_pictures()-1
     || get_current_position()<=her_sequence->get_start_position()))
  {
    set_new_current_position();
    //cout << "position : " << get_current_position() << endl;
    set_is_readed(true);
  }
}

void my_frame::set_current_position(long int new_position)
{
  if(new_position<her_sequence->get_nb_pictures() &&
     new_position>=0) //TODO : check min too
  {
    her_sequence->set_current_position(new_position);
    set_slider_position(new_position);
  }
}

void my_frame::set_current_position(type_of_current_reading cr)
{
  switch (cr)
  {
  case PLAY_FORWARD:
    //    if(get_current_position()+1<her_sequence->get_nb_pictures())
    set_current_position(get_current_position()+1);
    break;
  case PLAY_BACKWARD:
    //if(get_current_position()>=/*her_sequence->get_start_position()*/0)
    set_current_position(get_current_position()-1);
    break;
  case STOP:
    break;
  case IMAGE_BACKWARD:
    //if(get_current_position()>=/*her_sequence->get_start_position()*/0)
    set_current_position(get_current_position()-1);
    break;
  case IMAGE_FORWARD:
    //if(get_current_position()+1<her_sequence->get_nb_pictures())
    set_current_position(get_current_position()+1);
    break;
  case UN_INITIALIZED:
    break;
  }
  if(get_current_position()>=/*her_sequence->get_start_position()*/0 &&
     get_current_position()<her_sequence->get_nb_pictures()-1)
  set_slider_position(get_current_position());
}

void my_frame::set_new_current_position()
{
  switch(get_type_of_current_reading())
  {
  case PLAY_FORWARD:
    //if(get_current_position()<my_mother->get_position_max()-1)

    if(get_current_position()<her_sequence->get_nb_pictures()-1
       /*&& get_current_position()>=her_sequence->get_start_position()*/)
    {
      set_current_position(get_current_position()+1);
    }
    else
    {
      switch(get_play_mode())
      {
      case STOP_AT_THE_END:
	break;
      case LOOP_BACK:
	set_current_position(her_sequence->get_start_position());
	break;
      case PING_PONG:
	if(this!=my_mother->get_greater_frame())
	  {
	    if(
	        ( my_mother->get_greater_frame()->get_current_position()==
	       get_current_position()-1 ||
		my_mother->get_greater_frame()->get_current_position()==
		get_current_position())
		&&
	       my_mother->get_greater_frame()->get_type_of_current_reading()==
	       PLAY_BACKWARD)
	    {
	      set_current_position( my_mother->get_greater_frame()->get_current_position());
	      //set_current_position(PLAY_BACKWARD);
	      set_local_type_of_current_reading(PLAY_BACKWARD);
	    }
	  }
	else
	{
	  set_current_position(PLAY_BACKWARD);
	  set_local_type_of_current_reading(PLAY_BACKWARD);
	}
	break;
      }
    }

    break;
  case PLAY_BACKWARD:
    if(get_current_position()>her_sequence->get_start_position())
    {
      set_current_position(get_current_position()-1);
    }
    else
    {
      switch(get_play_mode())
      {
      case STOP_AT_THE_END:
	break;
      case LOOP_BACK:
	set_current_position(her_sequence->get_nb_pictures()-1);
	break;
      case PING_PONG:
	if(this!=my_mother->get_minus_frame())
	  {
	    if(
	       (my_mother->get_minus_frame()->get_current_position()==
		get_current_position() ||
		my_mother->get_minus_frame()->get_current_position()==
		get_current_position()+1)&&
	       my_mother->get_minus_frame()->get_type_of_current_reading()==
	       PLAY_FORWARD)
	    {
	      //set_current_position(PLAY_BACKWARD);
	      set_current_position( my_mother->get_greater_frame()->get_current_position());
	      set_local_type_of_current_reading(PLAY_FORWARD);
	    }
	  }
	else
	{
	  set_current_position(PLAY_FORWARD);
	  set_local_type_of_current_reading(PLAY_FORWARD);
	}
	break;
      }
    }
    break;
  case IMAGE_BACKWARD:
  case IMAGE_FORWARD:
  case STOP:
  case UN_INITIALIZED:
    break;
  }
}


void my_frame::set_local_type_of_current_reading(type_of_current_reading cr)
{
  local_type_of_current_reading=cr;
}

void my_frame::set_is_readed(bool val)
{
  is_readed=val;
}

sequence* my_frame::get_sequence()
{
  return her_sequence;
}

type_of_current_reading my_frame::get_type_of_current_reading()
{
  return local_type_of_current_reading;
}

type_of_current_reading my_frame::get_mother_type_of_current_reading()
{
  return my_mother->get_type_of_current_reading();
}

bool my_frame::get_is_readed() const
{
  return is_readed;
}

int long my_frame::get_zoom() const
{
  return zoom;
}

int my_frame::get_current_position() const
{
  return her_sequence->get_current_position();
}

play_mode my_frame::get_play_mode() const
{
  return my_mother->get_play_mode();
}

void my_frame::OnCloseWindow(wxCloseEvent& )
{
  my_mother->remove_frame(*this);
}

void my_frame::OnSlider( wxCommandEvent& WXUNUSED(event))
{
  set_current_position(m_slider->GetValue());
  set_slider_position(get_current_position());
}


void my_frame::OnPaint(wxPaintEvent& WXUNUSED(event))
{
}

void my_frame::OnSpin(wxSpinEvent& )
{
  set_current_position(child_spinbutton_position->GetValue());
}

void my_frame::OnComboZoom(wxCommandEvent& event)
{
  long int new_zoom;
  event.GetString().ToLong(&new_zoom);
  if(new_zoom!=1)
  {
    if(new_zoom!=zoom)
    {
      her_buffer->set_calcul(2);
    }
    else
    {
      her_buffer->set_calcul(1);
    }
  }
  else
  {
    her_buffer->set_calcul(0);
  }
  zoom=new_zoom;
  this->SetSize(her_sequence->get_width()*zoom,
		her_sequence->get_height()*zoom+20/**1.15*/);
}

void my_frame::set_slider_position(int position)
{
  if(position>=her_sequence->get_start_position())
    m_slider->SetValue(position);
}

void my_frame::OnSaveImage()
{
  wxImage *im;
  im=her_sequence->load(get_current_position());

  my_dialog_save dialog_save(this,"enregistrer image",
			   wxPoint(100, 100),
			   wxSize(300, 100));
  if(dialog_save.ShowModal()==wxID_OK)
  {
    im->SaveFile(dialog_save.name_of_file,
		 "image/" +  dialog_save.name_of_file.AfterLast('.'));
  }
}

BEGIN_EVENT_TABLE(my_frame, wxFrame)
  EVT_CLOSE(my_frame::OnCloseWindow)
  EVT_COMMAND_SCROLL_THUMBTRACK(ID_SLIDER,my_frame::OnSlider)
  EVT_PAINT(my_frame::OnPaint)

  EVT_SPIN_UP(ID_SPIN_BUTTON_POSITION,my_frame::OnSpin)
  EVT_SPIN_DOWN(ID_SPIN_BUTTON_POSITION,my_frame::OnSpin)
  EVT_COMBOBOX(ID_BUTTON_ZOOM, my_frame::OnComboZoom)
  EVT_BUTTON(ID_BUTTON_SAVE, my_frame::OnSaveImage)
END_EVENT_TABLE()
