/*
 *Copyright (C) 2001  LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>

#include <wx/file.h>

#include "image/lectureS.xpm"
#include "image/lectureR.xpm"
#include "image/arret.xpm"
#include "image/lecture.xpm"
#include "image/lectureF.xpm"

#include "sequence.hh"
#include "sequence_yuv.hh"
#include "sequence_pgm.hh"
#include "sequence_wxformat.hh"
#include "my_main_frame.hh"
#include "my_dialog_yuv.hh"
#include "my_application.hh"

my_main_frame::my_main_frame(wxFrame* parent, const wxString& title,
			     const wxPoint& pos, const wxSize& size,
			     my_application *ap) :
  wxFrame(parent,-1,title,pos,size),
  her_application(ap),
  the_timer(this,ID_TIMER),
  menu(0),
  video_control_tbar(0),
  current_reading(PLAY_FORWARD),
  spinbutton_with_combo_timer(0),
  combo_timer(0),
  speed_of_sequence(15),
  slider_position(0),
  greater_frame(0),
  minus_frame(0),
  her_play_mode(PING_PONG),
  play_mode_ping_pong_cbox(0),
  play_mode_stop_at_the_end_cbox(0),
  play_mode_loop_back_cbox(0),
  position_max(0),
  position_min(0),
  nb_pictures_greater_sequence(0),
  last_sequence_path("")
{
  // we start by default in forward play mode
  OnPlayForward();
}

void my_main_frame::create_video_control_tbar()
{
  video_control_tbar = CreateToolBar(common_tool_bar_style, ID_TBV);
  wxBitmapButton *temp_button=0;
  wxBitmap toolBarBitmaps[5];

  toolBarBitmaps[0] = wxBitmap(lectureS_xpm);
  toolBarBitmaps[1] = wxBitmap(lectureR_xpm);
  toolBarBitmaps[2] = wxBitmap(arret_xpm);
  toolBarBitmaps[3] = wxBitmap(lecture_xpm);
  toolBarBitmaps[4] = wxBitmap(lectureF_xpm);

  temp_button=new wxBitmapButton(video_control_tbar,
				 ID_TBV_IMAGE_BACKWARD,
				 toolBarBitmaps[0],
				 wxDefaultPosition);
  temp_button->SetToolTip("Retour en arrire d'une image");
  video_control_tbar->AddControl(temp_button);

  temp_button=new wxBitmapButton(video_control_tbar,
				 ID_TBV_PLAY_BACKWARD,
				 toolBarBitmaps[1],
				 wxDefaultPosition);
  temp_button->SetToolTip("Lecture en sens inverse");
  video_control_tbar->AddControl(temp_button);


  temp_button=new wxBitmapButton(video_control_tbar,
				 ID_TBV_STOP,
				 toolBarBitmaps[2],
				 wxDefaultPosition);
  temp_button->SetToolTip("Arrt de la sequence");
  video_control_tbar->AddControl(temp_button);


  temp_button=new wxBitmapButton(video_control_tbar,
				 ID_TBV_PLAY_FORWARD,
				 toolBarBitmaps[3],
				 wxDefaultPosition);
  temp_button->SetToolTip("Lecture de la sequence");
  video_control_tbar->AddControl(temp_button);

  temp_button=new wxBitmapButton(video_control_tbar,
				 ID_TBV_IMAGE_FORWARD,
				 toolBarBitmaps[4],
				 wxDefaultPosition);
  temp_button->SetToolTip("Avancer d'une image");
  video_control_tbar->AddControl(temp_button);
}

void my_main_frame::create_speed_controls()
{
  (void)new wxStaticBox(this, -1, "&Vitesse", wxPoint(10,10),
			 wxSize(90,40));
  //combo_timer= new wxComboBox(this,ID_COMBO_VITESSE_SEQUENCE);

  combo_timer=new wxComboBox(this, ID_COMBO_VITESSE_SEQUENCE,
			     "",wxDefaultPosition,
			     wxDefaultSize, number_of_possible_speed,
			     possible_speed, wxCB_READONLY ,
			     wxDefaultValidator,"comboBox");


  //for(int i=0;i<number_of_possible_speed;i++)
  //{
  // combo_timer->Append(possible_speed[i]);
  //}

  combo_timer->SetSize(30,25,55,20,-1);

  spinbutton_with_combo_timer = new wxSpinButton(this, ID_SPIN_SPEED);
  spinbutton_with_combo_timer->SetSize(15,25,20,20,-1);
  spinbutton_with_combo_timer->SetRange(1,35);
  spinbutton_with_combo_timer->SetValue(speed_of_sequence);

  (void)new wxStaticBox(this, -1, "&Position", wxPoint(10,60),
			 wxSize(200,48));

  slider_position=new wxSlider( this, ID_SLIDER_POSITION,
				0, 0,300,
				wxPoint(30,70), wxSize(170,-1),
				wxSL_LABELS );

  (void)new wxStaticBox(this, -1, "&Mode de lecture", wxPoint(10,120),
			 wxSize(200,110));
  play_mode_ping_pong_cbox=new wxCheckBox(this,ID_PING_PONG_CBOX,
					  "ping-pong",wxPoint(30,140),
					  wxDefaultSize,wxSUNKEN_BORDER,
					  wxDefaultValidator,
					  "checkbox");

  play_mode_ping_pong_cbox->SetValue(true);
  play_mode_stop_at_the_end_cbox=new wxCheckBox(this,ID_STOP_END_CBOX,
					  "stopper  la fin",wxPoint(30,170),
					  wxDefaultSize,wxSUNKEN_BORDER,
					  wxDefaultValidator,
					  "checkbox");

  play_mode_loop_back_cbox=new wxCheckBox(this,ID_LOOP_BACK_CBOX,
					  "en boucle",wxPoint(30,200),
					  wxDefaultSize,wxSUNKEN_BORDER,
					  wxDefaultValidator,
					  "checkbox");
}

void my_main_frame::create_menu()
{
  CreateStatusBar();
  wxMenu *file_menu    = new wxMenu;
  wxMenu *help_menu    = new wxMenu;
  wxMenu *options_menu = new wxMenu;
  file_menu->Append(ID_MENU_OPEN,
		    "&Ouvrir une sequence\tCtrl-O",
		    "Ouvrir une nouvelle sequence video");

  file_menu->Append(ID_MENU_QUIT,
		    "&Quitter\tCtrl-Q",
		    "Quitter le visualisateur");

  /* options_menu->Append(ID_MENU_OPTION_PLAY_MODE,
		       "&Mode de lecture",
		       "Fixer le mode de lecture");*/

  help_menu->Append(ID_MENU_ABOUT,
		    "&A propos\tCtrl-A",
		    "Information sur le Visualisateur");

  menu= new wxMenuBar(wxMB_DOCKABLE);
  menu->Append(file_menu, "&Fichier");
  menu->Append(options_menu, "&Options");
  menu->Append(help_menu, "&Aide");
  SetMenuBar(menu);
}

// TODO : necessary to stop the timer ?

void my_main_frame::OnPlayForward()
{
  if(the_timer.IsRunning())
  {
    the_timer.Stop();
  }
  set_current_reading(PLAY_FORWARD);
  TimerStart(speed_of_sequence);
}

void my_main_frame::OnImageForward()
{
  if(the_timer.IsRunning())
  {
    the_timer.Stop();
  }
  set_current_reading(IMAGE_FORWARD);
  TimerStart(speed_of_sequence);
}

void my_main_frame::OnPlayBackward()
{
  if(the_timer.IsRunning())
  {
    the_timer.Stop();
  }
  set_current_reading(PLAY_BACKWARD);
  TimerStart(speed_of_sequence);
}

void my_main_frame::OnImageBackward()
{
  if(the_timer.IsRunning())
  {
    the_timer.Stop();
  }
  set_current_reading(IMAGE_BACKWARD);
  TimerStart(speed_of_sequence);
}

void my_main_frame::OnStop()
{
  if(the_timer.IsRunning())
  {
    set_current_reading(STOP);
    the_timer.Stop();
  }
}

void my_main_frame::OnOpen()
{
  vector<wxString> vector_of_filename_sequence;
  wxFileDialog dialog(this, "Ouvrir squence", last_sequence_path, "",
		      "fichier YUV (*.yuv)|*yuv|fichier JPEG (*.jpg)|*.jpg|fichier PGM (*.pgm)|*.pgm|fichier PPM (*.ppm)|*.ppm|fichier PNG (*.png)|*.png|fichier XPM (*.xpm)|*.xpm|fichier BMP (*.bmp)|*.bmp", 0);

  if(dialog.ShowModal() == wxID_OK)
  {
    if(dialog.GetPath()== "")
    {
      return;
    }
    // sequences whose size is stored in the file(s)
    else if(dialog.GetPath().Contains(".pgm") ||
	    dialog.GetPath().Contains(".png") ||
	    dialog.GetPath().Contains(".jpg"))
    {
      vector_of_filename_sequence=get_find_files(dialog.GetPath());
      her_application->find_sequence_extension_and_create_new_frame(vector_of_filename_sequence,-1,-1,10);
      last_sequence_path=dialog.GetDirectory();
    }
    // sequences whose size is unknown
    else if(dialog.GetPath().Contains(".yuv"))
    {
      vector_of_filename_sequence.push_back(dialog.GetPath());
      my_dialog_yuv dialog_yuv(this, dialog.GetFilename(),
			       wxPoint(100, 100),
			       wxSize(300, 200));
      if(dialog_yuv.ShowModal()==wxID_OK)
      {
	long int width =0;
	long int height=0;
	dialog_yuv.width.ToLong(&width);
	dialog_yuv.height.ToLong(&height);
	cout << "width : " << width << " height : " << height << endl;
	if(height!=0 && width!=0)
	  her_application->find_sequence_extension_and_create_new_frame(vector_of_filename_sequence,width,height,10);
	last_sequence_path=dialog.GetDirectory();
      }
    }
  }
  cout << last_sequence_path << endl;
}

void my_main_frame::OnQuit()
{
  the_timer.Stop();
  for(int g=0;g<her_application->get_nb_frames();g++)
  {
    delete her_application->get_frame(g);
  }
  Close(TRUE);
}

void my_main_frame::OnAbout()
{
  (void)wxMessageBox(version_string,
		     "A Propos", wxOK);
}

void my_main_frame::OnSpin(wxSpinEvent& )
{
  printf("speed: %d %d\n", speed_of_sequence,
	  spinbutton_with_combo_timer->GetValue());
  speed_of_sequence=spinbutton_with_combo_timer->GetValue();
  combo_timer->SetValue(wxString::Format("%d", speed_of_sequence));
  set_speed_of_sequence_now(speed_of_sequence);
}


void my_main_frame::OnComboVitesse(wxCommandEvent& event)
{
  long int speed_temp = 0;
  if(event.GetString().ToLong(&speed_temp))
  {
    for(int i=0;i<number_of_possible_speed;i++)
    {
      if(event.GetString()!=possible_speed[i])
      {
	speed_of_sequence=speed_temp;
	set_speed_of_sequence_now(speed_of_sequence);
      }
    }
  }
  else if(event.GetString().ToLong(&speed_temp)) //TODO : clean up
  {
    if(speed_temp>0 && speed_temp<35) // TODO possible_speed
    {
      speed_of_sequence=speed_temp;
      set_speed_of_sequence_now(speed_of_sequence);
    }
  }
  spinbutton_with_combo_timer->SetValue(speed_of_sequence);
}

void my_main_frame::OnTimer(wxTimerEvent& )
{
  if(all_readed() || get_type_of_current_reading()==IMAGE_FORWARD
     || get_type_of_current_reading()==IMAGE_BACKWARD)
  {
    // display all loaded images
    for(int g=0;g<her_application->get_nb_frames();g++)
    {
      her_application->get_frame(g)->draw_bitmap();
    }
  }
  else
  {
    // loop over images to see which ones are available
    for(int g=0;g<her_application->get_nb_frames();g++)
    {
      my_frame* f=her_application->get_frame(g);
      if(!f->get_is_readed())
      {
	f->draw_bitmap();
      }
    }
  }
  if(greater_frame!=0)
  {
    slider_position->SetValue(greater_frame->get_current_position());
  }
  //cout<<"TIMER"<<endl;
}

void my_main_frame::OnCloseWindow(wxCloseEvent& )
{
  the_timer.Stop();
  for(int g=0;g<her_application->get_nb_frames();g++)
  {
    if(her_application->get_frame(g)!=0)
      delete her_application->get_frame(g);
  }
  cout << "END" << endl;
  this->Destroy();
  //Close(TRUE);
}

bool my_main_frame::all_readed()
{
  my_frame *i=0;
  int all_ok=0;
  for(int g=0;g<her_application->get_nb_frames();g++)
  {
    i=her_application->get_frame(g);
    if(i->get_is_readed())
      all_ok++;
  }

  return all_ok==her_application->get_nb_frames();
}

void my_main_frame::set_speed_of_sequence_now(int speed)
{
  if(!the_timer.IsRunning())
  {
    speed_of_sequence=speed;
  }
  else
  {
    the_timer.Stop();
    TimerStart(speed);
  }
}

void my_main_frame::set_current_reading(type_of_current_reading cr)
{
  my_frame* f=0;
  current_reading=cr;
  for(int g=0;g<her_application->get_nb_frames();g++)
  {
    f=her_application->get_frame(g);
    f->set_local_type_of_current_reading(cr);
    f->set_current_position(cr);
  }

  if(cr==STOP)
  {
    int position;
    if(get_greater_frame()!=0)
    {
      position=get_greater_frame()->get_current_position();
      for(int l=0;l<her_application->get_nb_frames();l++)
      {
	  f=her_application->get_frame(l);
	  if(f->get_current_position()==get_greater_frame()->get_current_position()+1 ||
	     f->get_current_position()==get_greater_frame()->get_current_position()-1 ||
	     f->get_current_position()==get_greater_frame()->get_current_position()-2 ||
	     f->get_current_position()==get_greater_frame()->get_current_position()+2 )
	  f->set_current_position(position);
	  //cout << "greater frame position : " << position << endl;
      }
    }
  }
}

void my_main_frame::TimerStart(int vitesse)
{
  the_timer.Start(1000/vitesse);
}

void my_main_frame::TimerStop()
{
  the_timer.Stop();
}

type_of_current_reading my_main_frame::get_type_of_current_reading() const
{
  return current_reading;
}

play_mode my_main_frame::get_play_mode() const
{
  return her_play_mode;
}

void my_main_frame::OnSliderPosition( wxCommandEvent& WXUNUSED(event))
{
  for(int g=0;g<her_application->get_nb_frames();g++)
    {
      her_application->get_frame(g)->set_current_position(slider_position->GetValue());
      //cout << "OnSliderPosition "
      //     <<  her_application->get_frame(g)->get_current_position() << endl;
    }
}

void my_main_frame::set_play_mode(play_mode pm)
{
  her_play_mode=pm;
}

void my_main_frame::OnCheckBox_ping_pong()
{
  play_mode_ping_pong_cbox->SetValue(true);
  play_mode_stop_at_the_end_cbox->SetValue(false);
  play_mode_loop_back_cbox->SetValue(false);
  set_play_mode(PING_PONG);
}

void my_main_frame::OnCheckBox_stop_end()
{
  play_mode_ping_pong_cbox->SetValue(false);
  play_mode_stop_at_the_end_cbox->SetValue(true);
  play_mode_loop_back_cbox->SetValue(false);
  set_play_mode(STOP_AT_THE_END);
}

void my_main_frame::OnCheckBox_loop_back()
{
  play_mode_ping_pong_cbox->SetValue(false);
  play_mode_stop_at_the_end_cbox->SetValue(false);
  play_mode_loop_back_cbox->SetValue(true);
  set_play_mode(LOOP_BACK);
}

void my_main_frame::remove_frame(my_frame& frame)
{
  her_application->remove_frame(frame);
}

my_frame* my_main_frame::get_greater_frame()
{
  return greater_frame;
}

void my_main_frame::set_greater_frame(my_frame* frame)
{
  greater_frame=frame;
}

my_frame* my_main_frame::get_minus_frame()
{
  return minus_frame;
}

void my_main_frame::set_minus_frame(my_frame* frame)
{
  minus_frame=frame;
}

void my_main_frame::set_position_max(int position)
{
  position_max=position;
}

void my_main_frame::set_position_min(int position)
{
  position_min=position;
}

int my_main_frame::get_position_max() const
{
  return position_max;
}

int my_main_frame::get_speed() const
{
  return speed_of_sequence;
}

int my_main_frame::get_position_min() const
{
  return position_min;
}

wxSlider* my_main_frame::get_slider_position()
{
  return slider_position;
}

vector<wxString> my_main_frame::get_find_files(wxString file) const
{
  vector<wxString> filenames;
  wxString number;
  wxFile file_test;
  wxString numeric;
  wxString numeric2;
  wxString next_file;
  long num;
  bool file_exist(true);
  for(int i=0;i<(int)file.Len();i++)
  {
    if(file.SubString(i,i).IsNumber())
    {
      number+=file.SubString(i,i);
    }
  }
  number.ToLong(&num);
  filenames.push_back(file);

  next_file=file;
  while(file_exist)
  {
    numeric.Printf("%d",num);
    numeric2.Printf("%d",++num);
    cout << numeric << endl;
    cout << numeric2 << endl;
    if(num==10)
    {
      next_file.Remove(next_file.Find('0'),1);
    }
    if(num==100)
    {
      next_file.Remove(next_file.Find("00"),1);
    }
    if(num==1000)
    {
      next_file.Remove(next_file.Find("000"),1);
    }
    next_file.Replace(numeric,numeric2);
    cout << next_file << endl;
    if(!file_test.Exists(next_file))
    {
      file_exist=false;
    }
    else
    {
      filenames.push_back(next_file);
    }
  }
  return filenames;
}

BEGIN_EVENT_TABLE(my_main_frame, wxFrame)
  EVT_BUTTON(ID_TBV_PLAY_FORWARD,my_main_frame::OnPlayForward)
  EVT_BUTTON(ID_TBV_PLAY_BACKWARD,my_main_frame::OnPlayBackward)
  EVT_BUTTON(ID_TBV_IMAGE_FORWARD,my_main_frame::OnImageForward)
  EVT_BUTTON(ID_TBV_IMAGE_BACKWARD,my_main_frame::OnImageBackward)
  EVT_BUTTON(ID_TBV_STOP,my_main_frame::OnStop)
  EVT_MENU(ID_MENU_QUIT,my_main_frame::OnCloseWindow)
  EVT_MENU(ID_MENU_OPEN,my_main_frame::OnOpen)
  EVT_MENU(ID_MENU_ABOUT,my_main_frame::OnAbout)

  EVT_COMBOBOX(ID_COMBO_VITESSE_SEQUENCE, my_main_frame::OnComboVitesse)

  EVT_TIMER(ID_TIMER,my_main_frame::OnTimer)

  EVT_SPIN_UP(ID_SPIN_SPEED,my_main_frame::OnSpin)
  EVT_SPIN_DOWN(ID_SPIN_SPEED,my_main_frame::OnSpin)
  EVT_COMMAND_SCROLL_THUMBTRACK(ID_SLIDER_POSITION,my_main_frame::OnSliderPosition)
  EVT_CHECKBOX(ID_PING_PONG_CBOX,my_main_frame::OnCheckBox_ping_pong)
  EVT_CHECKBOX(ID_STOP_END_CBOX,my_main_frame::OnCheckBox_stop_end)
  EVT_CHECKBOX(ID_LOOP_BACK_CBOX,my_main_frame::OnCheckBox_loop_back)
  EVT_CLOSE(my_main_frame::OnCloseWindow)
END_EVENT_TABLE()
