/*
 *Copyright (C) 2001  LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef my_main_frame__
#define my_main_frame__

#include <vector>

#include <wx/wxprec.h>
#include <wx/wx.h>
#include <wx/toolbar.h>
#include <wx/log.h>
#include <wx/image.h>
#include <wx/spinbutt.h>

#include "my_frame.hh"
#include "my_application.hh"
#include "common_type.hh"

using namespace std;

class sequence;
class my_application;

class my_main_frame : public wxFrame
{
private:
  my_application *her_application;

  wxTimer the_timer;

  // widgets

  wxMenuBar *menu;

  wxToolBar *video_control_tbar;
  type_of_current_reading current_reading;

  wxSpinButton *spinbutton_with_combo_timer;
  wxComboBox *combo_timer;
  int speed_of_sequence;

  wxSlider* slider_position;
  my_frame* greater_frame;
  my_frame* minus_frame;

  play_mode her_play_mode;
  wxCheckBox* play_mode_ping_pong_cbox;
  wxCheckBox* play_mode_stop_at_the_end_cbox;
  wxCheckBox* play_mode_loop_back_cbox;

  // min and max index of sequences
  // slider bounds
  int position_max;
  int position_min;
  int nb_pictures_greater_sequence;

  // last dirtectory visited from the fileselector
  wxString last_sequence_path;

  enum id_of_all_control{ID_TBV, ID_TBV_PLAY_FORWARD,
			 ID_TBV_PLAY_BACKWARD,
			 ID_TBV_IMAGE_FORWARD,
			 ID_TBV_IMAGE_BACKWARD,
			 ID_TBV_STOP,
			 ID_MENU_OPEN,
			 ID_MENU_QUIT,
			 ID_MENU_ABOUT,
			 ID_TBV_SHARP,
			 ID_COMBO_VITESSE_SEQUENCE,
			 ID_TIMER,
			 ID_SPEED_CTRL,
			 ID_SPIN_SPEED,
			 ID_SHOW_TBAR_DAUTHERS,
			 ID_SLIDER_POSITION,
			 ID_MENU_OPTION_PLAY_MODE,
			 ID_MENU_OPTION_SIZE,
			 ID_PING_PONG_CBOX,
			 ID_STOP_END_CBOX,
			 ID_LOOP_BACK_CBOX};

public:

  my_main_frame(wxFrame* parent, const wxString& title,
		const wxPoint& pos, const wxSize& size,my_application *ap);

  void create_video_control_tbar();
  void create_speed_controls();
  void create_menu();

  void OnPlayForward();
  void OnImageForward();
  void OnPlayBackward();
  void OnImageBackward();
  void OnStop();
  void OnQuit();
  void OnOpen();
  void OnAbout();
  void OnSpin(wxSpinEvent& event);
  void OnComboVitesse(wxCommandEvent& event);
  void OnTimer(wxTimerEvent& event);
  void OnSliderPosition(wxCommandEvent& WXUNUSED(event));
  void OnCheckBox_ping_pong();
  void OnCheckBox_stop_end();
  void OnCheckBox_loop_back();
  void OnCloseWindow(wxCloseEvent& event);

  void set_speed_of_sequence_now(int speed);
  void set_current_reading(type_of_current_reading cr);
  void set_position_max(int position);
  void set_position_min(int position);
  void set_play_mode(play_mode pm);
  void set_greater_frame(my_frame* frame);
  void set_minus_frame(my_frame* frame);

  int convert_speed_interval(int speed);
  void TimerStart(int vitesse);
  void TimerStop();
  bool all_readed();

  void remove_frame(my_frame& frame);

  // TODO const
  type_of_current_reading get_type_of_current_reading() const;
  play_mode get_play_mode() const;
  my_frame* get_greater_frame();
  my_frame* get_minus_frame();
  int get_speed() const;
  int get_position_max() const;
  int get_position_min() const;
  wxSlider* get_slider_position();
  vector<wxString> get_find_files(wxString file) const;

  DECLARE_EVENT_TABLE()
};

#endif
