/*
 *Copyright (C) 2001   LIS Laboratory Yann Davin
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <vector>
//#include <stdlib.h>

#include <wx/wx.h>
#include <wx/file.h>

#include "sequence_yuv.hh"


sequence_yuv::sequence_yuv(vector<wxString> filenames,
				 int width,
				 int height) :
  sequence(filenames)
{
  this->width=width;
  this->height=height;
  size=calcul_size();
  nb_pictures=calcul_nb_pictures();
  start_position=0;
  final_position=nb_pictures-1;
}

int sequence_yuv::calcul_nb_pictures()
{
  int size_tmp;
  size_tmp=get_size();
  size=size/3 + size/3;
  return (size/(get_width()*get_height()));
}

//real == float or double
template<class real>
static inline unsigned char round_truncate(real x)
{
  int tmp=int(x+0.5);
  if(tmp<0)
    return 0;
  if(tmp>=255)
    return 255;
  return tmp;
}

static inline void convert_pixel(unsigned char* ptr_rgb, unsigned char y, unsigned char u,unsigned char v)
{
  *ptr_rgb++=round_truncate(1.164*(y-16)+1.596*(v-128));
  *ptr_rgb++=round_truncate(1.164*(y-16)-0.813*(v-128) -0.391*(u-128));
  *ptr_rgb++=round_truncate(1.164*(y-16)+2.018*(u-128));
}

wxImage* sequence_yuv::load(int position)
{
  FILE *file;
  wxImage *image;
  file=fopen(get_name(0).c_str(),"rb");
  //cout << get_name(0) << endl;
  long int taille=get_height()*get_width();
  vector<unsigned char> Y(taille);
  vector<unsigned char> U(taille/4);
  vector<unsigned char> V(taille/4);
  unsigned char* tab=(unsigned char*)malloc(taille*3);
  fseek(file, (taille+taille/2)*position, SEEK_SET);

  //fread(&YUV[0], taille + taille/2,sizeof(unsigned char),file);
  fread(&Y[0], taille, sizeof(unsigned char), file);
  fread(&U[0], taille/4, sizeof(signed char), file);
  fread(&V[0], taille/4, sizeof(signed char), file);
  fclose(file);

  unsigned char* ptr_rgb=tab;
  unsigned char* ptr_y=&Y[0];

  for(int i=0;i<get_height()/2;i++)
  {
    for(int j=0;j<get_width()/2;j++)
    {
      unsigned char u=U[j+i*get_width()/2];
      unsigned char v=V[j+i*get_width()/2];

      convert_pixel(ptr_rgb,*ptr_y,u,v);
      ptr_y++;
      ptr_rgb+=3;

      convert_pixel(ptr_rgb,*ptr_y,u,v);
      ptr_y+=get_width();
      ptr_rgb+=3*get_width();

      convert_pixel(ptr_rgb,*ptr_y,u,v);
      ptr_y--;
      ptr_rgb-=3;

      convert_pixel(ptr_rgb,*ptr_y,u,v);
      ptr_y+=2-get_width();
      ptr_rgb+=6-3*get_width();
    }
    ptr_y+=get_width();
    ptr_rgb+=3*get_width();
  }

  // TRUE means that the wxImage destructor will not free the tab memory
  image = new wxImage(get_width(), get_height(), tab, TRUE);
  //cout << "pointeur image : " << image << endl;
  return image;
}

bool sequence_yuv::must_free_data_image() const
{
  return true;
}
